from pathlib import Path
import pandas as pd
import re

from .utility import first_match_or_empty, to_datetime_safe


METADATA_SCHEMA = sorted(["path", "from", "date"])


def make_email_metadata(email_path: str | Path) -> dict[str, str]:
    """Make an email metadata object by parsing the email contents

    Args:
        email_path:
        email_contents: Contents of an email file (from f.readlines())

    Returns: Dict containing the required metadata
    """
    key_is_set = {key: False for key in METADATA_SCHEMA}
    metadata = {"path": str(email_path)}
    key_is_set["path"] = True

    with open(email_path, "rb") as f:
        for line in f:
            try:
                k, v = [
                    val.strip()
                    for val in line.decode(errors="ignore").split(":", maxsplit=1)
                ]
                k = k.lower()
                if k in METADATA_SCHEMA:
                    metadata[k] = v
                    key_is_set[k] = True
                    if all([v for _, v in key_is_set.items()]):
                        break
            # If it's not a metadata line just pass
            except ValueError:
                pass

    # Validate that all keys have been found
    assert sorted(list(metadata.keys())) == METADATA_SCHEMA
    return metadata


def parse_maildir(path_to_dir: str | Path):
    """Parse all of the emails within the specified maildir box (not recursively)

    Args:
        path_to_dir: Path to maildir

    Returns: MailDir object initialized with email information

    """
    file_list = Path(path_to_dir).glob("*")
    email_metadata = [make_email_metadata(file) for file in file_list]
    return MailDir(email_metadata)


class TopSender:
    """Simple class to store the top sender,
    alongisde names they used and the count of emails sent by them
    """

    def __init__(self, email: str, names: list[str], count: int):
        self.email = email
        self.names = names
        self.count = count


class MailDir:
    """Representation of the local maildir repository.
    Stores the metadata associated with all local emails.
    """

    name_regex = r"^(.*?)(?=<)"
    email_regex = r"<?([a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+)>?"

    KEYS_AND_FUNCS = {
        "name": lambda df: df["from"].map(
            lambda x: first_match_or_empty(MailDir.name_regex, x).strip('" ')
        ),
        "email": lambda df: df["from"].map(
            lambda x: first_match_or_empty(MailDir.email_regex, x).strip("")
        ),
        "date": lambda df: df["date"].map(lambda x: to_datetime_safe(x)),
    }

    def __init__(self, email_metadata: list[dict[str, str]]):
        if email_metadata:
            self._df = pd.DataFrame(email_metadata)

            for k, func in self.KEYS_AND_FUNCS.items():
                self._df[k] = func(self._df)

        else:
            self._df = pd.DataFrame(
                columns=METADATA_SCHEMA + list(self.KEYS_AND_FUNCS.keys())
            )

    def get_top_n_senders(self, n: int) -> list[TopSender]:
        """Calculate the top n senders and returns their information as a TopSender object
        The TopSender object
        Args:
            n: Number of senders to retrieve

        Returns: list of TopSender objects

        """

        unique_senders = self._df["email"].value_counts().iloc[0:n]
        senders = [
            TopSender(
                email,
                list(self._df.loc[self._df["email"] == email, "name"].unique()),
                count,
            )
            for (email, count) in unique_senders.items()
        ]

        return senders

    def get_paths_for_email(self, email: str) -> list[str]:
        return self._df.loc[self._df.email == email, "path"].to_list()

    def remove_email(self, email: str):
        self._df.drop(self._df[self._df["email"] == email].index, inplace=True)
